"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useWindowSize = void 0;
var react_1 = require("react");
var useEventListener_1 = require("../useEventListener");
var useIsomorphicLayoutEffect_1 = require("../useIsomorphicLayoutEffect");
var IS_SERVER = typeof window === 'undefined';
/**
 * Custom hook that tracks the size of the window.
 * @param {?UseWindowSizeOptions} [options] - The options for customizing the behavior of the hook (optional).
 * @param {?boolean} [options.initializeWithValue] - If `true` (default), the hook will initialize reading the window size. In SSR, you should set it to `false`, returning `undefined` initially.
 * @returns {object} An object containing the width and height of the window.
 * @property {number} width - The width of the window.
 * @property {number} height - The height of the window.
 * @see [Documentation](https://usehooks-ts.com/react-hook/use-window-size)
 * @example
 * // Usage of useWindowSize hook
 * const { width, height } = useWindowSize();
 * console.log(`Window size: ${width} x ${height}`);
 */
function useWindowSize(options) {
    if (options === void 0) { options = {}; }
    var _a = options.initializeWithValue, initializeWithValue = _a === void 0 ? true : _a;
    if (IS_SERVER) {
        initializeWithValue = false;
    }
    var _b = (0, react_1.useState)(function () {
        if (initializeWithValue) {
            return {
                width: window.innerWidth,
                height: window.innerHeight,
            };
        }
        return undefined;
    }), windowSize = _b[0], setWindowSize = _b[1];
    var handleSize = function () {
        setWindowSize({
            width: window.innerWidth,
            height: window.innerHeight,
        });
    };
    // TODO: Prefer incoming useResizeObserver hook
    (0, useEventListener_1.useEventListener)('resize', handleSize);
    // Set size at the first client-side load
    (0, useIsomorphicLayoutEffect_1.useIsomorphicLayoutEffect)(function () {
        handleSize();
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    return windowSize;
}
exports.useWindowSize = useWindowSize;
